@@ -31,7 +31,7 @@
 */
 #define DO_RECEIVED	/* whether to add a "Received:" header */
 #define DO_GETPWUID	/* whether to try a getpwuid() if getlogin() fails */
-#define DO_MINUS_S	/* whether to implement the -s flag */
+#define DO_MINUS_SP	/* whether to implement the -s and -p flags */
 #define DO_DNS		/* whether to do a name lookup on -s, or just IP# */
 
 
@@ -64,9 +64,10 @@
 static char* argv0;
 static char* fake_from;
 static int parse_message, verbose;
-#ifdef DO_MINUS_S
+#ifdef DO_MINUS_SP
 static char* server;
-#endif /* DO_MINUS_S */
+static short port;
+#endif /* DO_MINUS_SP */
 static int timeout;
 static int sockfd1, sockfd2;
 static FILE* sockrfp;
@@ -109,9 +110,10 @@ main( int argc, char** argv )
     argv0 = argv[0];
     fake_from = (char*) 0;
     parse_message = 0;
-#ifdef DO_MINUS_S
+#ifdef DO_MINUS_SP
     server = "127.0.0.1";
-#endif /* DO_MINUS_S */
+    port = SMTP_PORT;
+#endif /* DO_MINUS_SP */
     verbose = 0;
     timeout = DEFAULT_TIMEOUT;
     argn = 1;
@@ -121,14 +123,18 @@ main( int argc, char** argv )
 	    fake_from = &(argv[argn][2]);
 	else if ( strcmp( argv[argn], "-t" ) == 0 )
 	    parse_message = 1;
-#ifdef DO_MINUS_S
+#ifdef DO_MINUS_SP
 	else if ( strncmp( argv[argn], "-s", 2 ) == 0 && argv[argn][2] != '\0' )
 	    server = &(argv[argn][2]);
-#endif /* DO_MINUS_S */
+	else if ( strncmp( argv[argn], "-p", 2 ) == 0 && argv[argn][2] != '\0' )
+	    port = atoi( &(argv[argn][2]) );
+#endif /* DO_MINUS_SP */
 	else if ( strncmp( argv[argn], "-T", 2 ) == 0 && argv[argn][2] != '\0' )
 	    timeout = atoi( &(argv[argn][2]) );
 	else if ( strcmp( argv[argn], "-v" ) == 0 )
 	    verbose = 1;
+	else if ( strcmp( argv[argn], "-i" ) == 0 )
+	    ;	/* ignore */
 	else if ( strcmp( argv[argn], "--" ) == 0 )
 	    ;	/* ignore */
 	else
@@ -164,6 +170,12 @@ main( int argc, char** argv )
 	else
 	    (void) snprintf( from, sizeof(from), "%s", fake_from );
 
+    /* Strip off any angle brackets in the from address. */
+    while ( from[0] == '<' )
+	(void) strcpy( from, &from[1] );
+    while ( from[strlen(from)-1] == '>' )
+	from[strlen(from)-1] = '\0';
+
     message = slurp_message();
 #ifdef DO_RECEIVED
     received = make_received( from, username, hostname );
@@ -205,7 +217,7 @@ main( int argc, char** argv )
 	exit( 1 );
 	}
 
-    (void) snprintf( buf, sizeof(buf), "MAIL FROM:%s", from );
+    (void) snprintf( buf, sizeof(buf), "MAIL FROM:<%s>", from );
     send_command( buf );
     status = read_response();
     if ( status != 250 )
@@ -267,16 +279,16 @@ main( int argc, char** argv )
 static void
 usage( void )
     {
-#ifdef DO_MINUS_S
+#ifdef DO_MINUS_SP
 #ifdef DO_DNS
-    char* sflag = "[-s<server>] ";
+    char* spflag = "[-s<server>] [-p<port>] ";
 #else /* DO_DNS */
-    char* sflag = "[-s<server_ip>] ";
+    char* spflag = "[-s<server_ip>] [-p<port>] ";
 #endif /* DO_DNS */
-#else /* DO_MINUS_S */
-    char* sflag = "";
-#endif /* DO_MINUS_S */
-    (void) fprintf( stderr, "usage:  %s [-f<name>] [-t] %s[-T<timeout>] [-v] [address ...]\n", argv0, sflag );
+#else /* DO_MINUS_SP */
+    char* spflag = "";
+#endif /* DO_MINUS_SP */
+    (void) fprintf( stderr, "usage:  %s [-f<name>] [-t] %s[-T<timeout>] [-v] [address ...]\n", argv0, spflag );
     exit( 1 );
     }
 
@@ -526,7 +538,16 @@ add_recipient( char* recipient, int len )
 	--len;
 	}
 
-    (void) snprintf( buf, sizeof(buf), "RCPT TO:%.*s", len, recipient );
+    /* Strip off any angle brackets. */
+    while ( len > 0 && *recipient == '<' )
+	{
+	++recipient;
+	--len;
+	}
+    while ( len > 0 && recipient[len-1] == '>' )
+	--len;
+
+    (void) snprintf( buf, sizeof(buf), "RCPT TO:<%.*s>", len, recipient );
     send_command( buf );
     status = read_response();
     if ( status != 250  && status != 251 )
@@ -563,7 +584,7 @@ open_client_socket( void )
 #ifdef USE_IPV6
 
     {
-#ifdef DO_MINUS_S
+#ifdef DO_MINUS_SP
     struct sockaddr_in sa4;
     struct addrinfo hints;
     char portstr[10];
@@ -572,25 +593,26 @@ open_client_socket( void )
     struct addrinfo* ai2;
     struct addrinfo* aiv4;
     struct addrinfo* aiv6;
-#endif /* DO_MINUS_S */
+#endif /* DO_MINUS_SP */
 
     sock_family = PF_INET6;
 
-#ifdef DO_MINUS_S
-    if ( inet_pton( PF_INET, server, (void*) &sa4.sin_addr ) == 0 )
+#ifdef DO_MINUS_SP
+    (void) memset( (void*) &sa4, 0, sizeof(sa4) );
+    if ( inet_pton( AF_INET, server, (void*) &sa4.sin_addr ) == 1 )
 	{
 	sock_family = PF_INET;
-	sa4.sin_port = htons( SMTP_PORT );
+	sa4.sin_port = htons( port );
 	sa_len = sizeof(sa4);
 	(void) memmove( &sa, &sa4, sa_len );
 	}
-    else if ( inet_pton( PF_INET6, server, (void*) &sa.sin6_addr ) == -1 )
+    else if ( inet_pton( AF_INET6, server, (void*) &sa.sin6_addr ) != 1 )
 	{
 #ifdef DO_DNS
 	(void) memset( &hints, 0, sizeof(hints) );
 	hints.ai_family = PF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
-	(void) snprintf( portstr, sizeof(portstr), "%d", SMTP_PORT );
+	(void) snprintf( portstr, sizeof(portstr), "%d", port );
 	if ( (gaierr = getaddrinfo( server, portstr, &hints, &ai )) != 0 )
 	    {
 	    (void) fprintf(
@@ -606,11 +628,11 @@ open_client_socket( void )
 	    {
 	    switch ( ai2->ai_family )
 		{
-		case AF_INET:
+		case PF_INET:
 		if ( aiv4 == (struct addrinfo*) 0 )
 		    aiv4 = ai2;
 		break;
-		case AF_INET6:
+		case PF_INET6:
 		if ( aiv6 == (struct addrinfo*) 0 )
 		    aiv6 = ai2;
 		break;
@@ -665,10 +687,10 @@ open_client_socket( void )
 	exit( 1 );
 #endif /* DO_DNS */
 	}
-#else /* DO_MINUS_S */
+#else /* DO_MINUS_SP */
     sa.sin6_addr = in6addr_any;
     sa.sin6_port = htons( SMTP_PORT );
-#endif /* DO_MINUS_S */
+#endif /* DO_MINUS_SP */
 
     sa.sin6_family = sock_family;
 
@@ -677,17 +699,17 @@ open_client_socket( void )
 #else /* USE_IPV6 */
 
     {
-#ifdef DO_MINUS_S
+#ifdef DO_MINUS_SP
     struct hostent *he;
-#else /* DO_MINUS_S */
+#else /* DO_MINUS_SP */
     char local_addr[4] = { 127, 0, 0, 1 };
-#endif /* DO_MINUS_S */
+#endif /* DO_MINUS_SP */
 
     sock_family = PF_INET;
-    sa.sin_port = htons( SMTP_PORT );
 
-#ifdef DO_MINUS_S
+#ifdef DO_MINUS_SP
     sa.sin_addr.s_addr = inet_addr( server );
+    sa.sin_port = htons( port );
     if ( (int32_t) sa.sin_addr.s_addr == -1 )
 	{
 #ifdef DO_DNS
@@ -707,9 +729,10 @@ open_client_socket( void )
 	exit( 1 );
 #endif /* DO_DNS */
 	}
-#else /* DO_MINUS_S */
+#else /* DO_MINUS_SP */
     (void) memmove( &sa.sin_addr, local_addr, sizeof(local_addr) );
-#endif /* DO_MINUS_S */
+    sa.sin_port = htons( SMTP_PORT );
+#endif /* DO_MINUS_SP */
 
     sa.sin_family = sock_family;
     }
